#include "C:\Users\Bogdan\Documents\Arduino\libraries\LiquidCrystal_I2C\LiquidCrystal_I2C.h"
#include <Arduino.h>

LiquidCrystal_I2C lcd_1(0x27, 16, 2);
int calibrationTime = 5; // Calibration time for the PIR sensor

volatile bool isOn = true; // Make isOn volatile since it will be accessed from an interrupt
bool lastState = false;
long unsigned int lowIn;
long unsigned int pause = 200; // Pause duration before assuming motion has stopped
boolean lockLow = true;
boolean takeLowTime = false;
int ledPin = 13;   // LED pin for visualization
int pirPin = 10;   // PIR sensor pin
int buzzerPin = 5; // Buzzer pin
int buttonPin = 2; // Button pin for interrupt

int RGB_RED_PIN = 6;
int RGB_GREEN_PIN = 7;

boolean systemActive = true;

// Debounce variables
volatile unsigned long lastDebounceTime = 0;
const unsigned long debounceDelay = 50; // 50 milliseconds debounce time

// Timer variables
volatile unsigned long lastActivityTime = 0; // Last time activity was detected
const unsigned long shutdownDelay = 15000;   // 15 seconds delay before shutdown

void toggleSystem()
{
  // Check if enough time has passed since the last press to debounce
  if ((millis() - lastDebounceTime) > debounceDelay)
  {
    isOn = !isOn;
    lastDebounceTime = millis(); // Update the last debounce time
    lastActivityTime = millis(); // Reset the activity timer
  }
}

void setup()
{
  Serial.begin(9600);
  pinMode(pirPin, INPUT);
  pinMode(ledPin, OUTPUT);
  pinMode(buzzerPin, OUTPUT);
  pinMode(RGB_RED_PIN, OUTPUT);
  pinMode(RGB_GREEN_PIN, OUTPUT);
  pinMode(buttonPin, INPUT_PULLUP); // Set button pin as input with pull-up resistor

  digitalWrite(pirPin, LOW);
  lcd_1.init();
  lcd_1.backlight();
  Serial.println("Hello World!");
  lcd_1.print("Starting...");
  pinMode(9, OUTPUT);

  // Attach interrupt to button pin
  attachInterrupt(digitalPinToInterrupt(buttonPin), toggleSystem, FALLING);

  // Timer1 configuration: 1Hz (1 second interval)
  cli();                      // Disable all interrupts
  TCCR1A = 0;                          // Set entire TCCR1A register to 0
  TCCR1B = 0;                          // Same for TCCR1B
  TCNT1 = 0;                           // Initialize counter value to 0
  OCR1A = 15624;                       // Compare match register (16MHz/1024/1Hz - 1)
  TCCR1B |= (1 << WGM12);              // CTC mode
  TCCR1B |= (1 << CS12) | (1 << CS10); // 1024 prescaler
  TIMSK1 |= (1 << OCIE1A);             // Enable timer compare interrupt
  sei();                        // Enable all interrupts

  // Give the sensor some time to calibrate
  Serial.print("Calibrating sensor ");
  for (int i = 0; i < calibrationTime; i++)
  {
    Serial.print(".");
    delay(1000);
  }
  Serial.println("\nDone");
  Serial.println("SENSOR ACTIVE");
  delay(50);
}

ISR(TIMER1_COMPA_vect)
{
  // Timer1 ISR: Increment seconds counter
  if (isOn && millis() - lastActivityTime >= shutdownDelay)
  {
    isOn = false;
  }
}

void loop()
{
  // Set RGB LED state
  if (isOn)
  {
    digitalWrite(RGB_RED_PIN, HIGH);
    digitalWrite(RGB_GREEN_PIN, LOW);
  }
  else
  {
    digitalWrite(RGB_RED_PIN, LOW);
    digitalWrite(RGB_GREEN_PIN, HIGH);
  }

  // Update LCD if state changed
  if (isOn != lastState)
  {
    lastState = isOn;
    lcd_1.clear();
    lcd_1.setCursor(0, 0);
    if (isOn)
    {
      lcd_1.print("System ON");
    }
    else
    {
      lcd_1.print("System OFF");
    }
  }

  // Handle PIR sensor
  if (isOn)
  {
    if (digitalRead(pirPin) == HIGH)
    {
      digitalWrite(buzzerPin, HIGH); // Turn on the buzzer
      if (lockLow)
      {
        lockLow = false;
        Serial.println("---");
        Serial.print("Motion detected at ");
        lcd_1.clear();
        lcd_1.print("Detected Movement");
        Serial.print(millis() / 1000);
        Serial.println(" sec");
        delay(50);
      }
      takeLowTime = true;
      lastActivityTime = millis(); // Reset the activity timer
    }

    digitalWrite(ledPin, digitalRead(pirPin));
    digitalWrite(buzzerPin, digitalRead(pirPin)); // Turn on the buzzer

    if (digitalRead(pirPin) == LOW)
    {
      digitalWrite(ledPin, LOW);    // Visualize sensor output state with LED
      digitalWrite(buzzerPin, LOW); // Turn off the buzzer
      if (takeLowTime)
      {
        lowIn = millis();
        takeLowTime = false;
      }
      if (!lockLow && millis() - lowIn > pause)
      {
        lockLow = true;
        Serial.print("Motion ended at ");
        Serial.print((millis() - pause) / 1000);
        Serial.println(" sec");
        lcd_1.clear();
        lcd_1.print("Movement Stopped");
        delay(50);
      }
    }
  }
}
